/* ----------------------------------------------------------------------------
 *
 * Raima Database Manager
 *
 * Copyright (c) 2012 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 *
 * ----------------------------------------------------------------------------
 */

#include <stdio.h>

/* rdm.h is required in all files using the RDM Database */
#include "rdm.h"

/* core02_db.h was generated from the DDL file core02_db.ddl by the ddlp.
 * utility.
 */
#include "core02_db.h"

/* core02_db_dbd.h was generated from the DDL file core02_db.ddl by the 
 * ddlp utility.  It is required to use the d_open_ptr API
 */
#include "core02_db_dbd.h"

/* EXAMPLE - Core for C */
/* Each time you run this example, a new record is created in the
 * database, then all existing records are read from the database
 * and printed.
 *
 * The DDL (Database Definition Language) is in core02_db.ddl. The database
 * schema is embedded into the application and the database will be created 
 * the first time the application is run.  Subsequent runs will use the now
 * existing database
 *
 * The following RDM Functions are illustrated:
 *      d_opentask/d_closetask
 *      d_open_ptr/d_close
 *      d_fillnew
 *      d_recfrst/d_recnext
 *      d_recread
 * They are fully documented in http://docs.raima.com/rdm/.
 *
 * Each of these functions returns an integer status code, where the
 * value S_OKAY indicates a successful call.
 *
 * For simplicity, this example does not check all return codes, but good
 * programming practices would dictate that they are checked after each
 * RDM call.
 *
 * The actual database will be stored in a directory named 'core02_db' in the
 * project directory.
 */


/* =======================================================================
*/
int32_t main(void)
{
    DB_TASK *task;
    int32_t  status;
    struct core02_record core02_input;
    struct core02_record core02_output;

    /* create the context for the database operations */
    d_opentask(&task);

    /* open the database for exclusive access, this will create the database
     * dictionary (core02_db.dbd file if it does not already exist.
     */
    status = d_open_ptr("core02_db", "x", &core02_db_dbd, sizeof(core02_db_dbd), task);
    if (status != S_OKAY)
    {
        fprintf (stderr, "Exclusive open failed, error = %s: %s\n", d_errorname(status), d_errorstr(status) );
        d_closetask (task);
        return (1);
    }

    /* Check to see if the database data file already exists, if not d_recfrst
     * will return S_NOFILE and we know we need to initialize the database.
     * Initializing the database will create the data files if they do not exist.
     * If the the data files do already exist the will be removed and new empty
     * files will be created.
     */
    status = d_recfrst(CORE02_RECORD, task, CURR_DB);
    if(status == S_NOFILE)
    {
        d_initialize(task, CURR_DB);
    }

    /* create a record in the new database */
    strncpy (core02_input.message, "Core02", SIZEOF_MESSAGE);
    status = d_fillnew (CORE02_RECORD, &core02_input, task, CURR_DB);

    /* scan and read all created records */
    /* one additional record will be created each time you run this example */
    for (status = d_recfrst (CORE02_RECORD, task, CURR_DB);
         status == S_OKAY;
         status = d_recnext (task, CURR_DB))
    {
        d_recread (&core02_output, task, CURR_DB);
        printf ("%s\n", core02_output.message);
    }

    /* close the database */
    d_close (task);
    d_closetask (task);
    printf ("Press <Enter> to continue . . .\n");
    getchar ();
}
